import { Prisma, PrismaClient } from '@prisma/client';
import {Uuid} from '@theoryofnekomata/uuid-buffer';
import {User} from '../user';
import {Log} from './models';

export interface LogService {
  create(subject: User, action: string, parameters?: Record<string, unknown>): Promise<Log>
}

export class LogServiceImpl implements LogService {
  private readonly prismaClient: PrismaClient

  constructor() {
    this.prismaClient = new PrismaClient()
  }

  async create(subject: User, action: string, parameters?: Record<string, unknown>): Promise<Log> {
    const createData: Prisma.LogCreateInput = {
      subjectUsername: subject.username,
      subjectUserId: subject.id,
      action,
    }

    if (parameters) {
      createData['parameters'] = {
        create: Object.entries(parameters).map(([key, value]) => ({
          key,
          value: `${value}`,
        })),
      }
    }

    const raw = await this.prismaClient.log.create({
      data: createData,
    })

    return {
      ...raw,
      subjectUserId: Uuid.from(raw.subjectUserId),
    }
  }
}
