import {PrismaClient} from '@prisma/client';
import {Org} from './models';
import {Uuid} from '@theoryofnekomata/uuid-buffer';
import {notFoundFactory} from '../../packages/prisma-error-utils';
import {OrgNotFoundError} from './responses';

export interface OrgService {
  getFromId(orgId: Org['id']): Promise<Org>;
}

export class OrgServiceImpl {
  private readonly prismaClient: PrismaClient;

  constructor() {
    this.prismaClient = new PrismaClient();
  }

  private static serialize<T extends Org>(org: T): T {
    return {
      ...org,
      id: Uuid.from(org.id),
      creatorUserId: Uuid.from(org.creatorUserId),
    };
  }

  async getFromId(id: Org['id']): Promise<Org> {
    const org = await this.prismaClient.org.findUnique({
      where: {
        id,
      },
      rejectOnNotFound: notFoundFactory(OrgNotFoundError),
    })

    return OrgServiceImpl.serialize(org);
  }
}
