import { RouteHandlerMethod } from 'fastify'
import { Controller } from 'src/packages/fastify-utils-theoryofnekomata'
import { UserService, UserServiceImpl } from 'src/modules/user/User.service'
import { RegisterUserFormData } from 'src/modules/user/models'
import { SessionService, SessionServiceImpl } from 'src/modules/auth'
import {
	UserRegisteredResponse,
	UnableToRegisterUserError,
} from './responses'

export interface UserController extends Controller<'register'> {
}

export class UserControllerImpl implements UserController {
	private readonly userService: UserService
	private readonly sessionService: SessionService

	constructor() {
		this.userService = new UserServiceImpl()
		this.sessionService = new SessionServiceImpl()
	}

	readonly register: RouteHandlerMethod = async (request, reply) => {
		try {
			const body = request.body as RegisterUserFormData
			const newUser = await this.userService.create({
				username: body.username,
				password: body.confirmNewPassword,
			})
			const newSession = await this.sessionService.create({
				userId: newUser.id,
			})

			reply.sendData(new UserRegisteredResponse(newSession))
		} catch (causeRaw) {
			const cause = causeRaw as Error
			throw new UnableToRegisterUserError('The operation could not be performed. Try again later.', { cause })
		}
	}
}
