import fp from 'fastify-plugin'
import { FastifyInstance, FastifyRequest } from 'fastify'

export interface FastifySessionOpts<SessionType = unknown, SessionId = string, UserType = unknown, RequestType extends FastifyRequest = FastifyRequest> {
	extractSessionId: (request: RequestType) => SessionId | null | undefined,
	isSessionValid: (sessionId: SessionId) => Promise<boolean>,
	getSession: (sessionId: SessionId) => Promise<SessionType>,
	getUserFromSession?: (session: SessionType) => Promise<UserType>,
	sessionRequestKey?: string,
	userRequestKey?: string,
}

const fastifySession = async (app: FastifyInstance, opts: FastifySessionOpts) => {
	const {
		sessionRequestKey = 'session',
		userRequestKey = 'user'
	} = opts;
	app.decorateRequest(sessionRequestKey, null);
	app.decorateRequest(userRequestKey, null);
	app.addHook('onRequest', async (request: FastifyRequest) => {
		const sessionId = opts.extractSessionId(request)
		if (typeof sessionId === 'string') {
			const isSessionValid = await opts.isSessionValid(sessionId)
			if (isSessionValid) {
				const mutableRequest = (request as unknown) as Record<string, unknown>
				const session = await opts.getSession(sessionId);
				mutableRequest[sessionRequestKey] = session;
				if (typeof opts.getUserFromSession === 'function') {
					mutableRequest[userRequestKey] = await opts.getUserFromSession(session);
				}
			}
		}
	})
}

export default fp(fastifySession)
